@@ -107,11 +107,16 @@ static int scanstdin(const struct cl_engine *engine, const struct cl_limits *lim
 
     if(!(fs = fopen(file, "wb"))) {
 	logg("!Can't open %s for writing\n", file);
+	free(file);
 	return 63;
     }
 
     while((ret = fread(buff, 1, FILEBUFF, stdin)))
-	fwrite(buff, 1, ret, fs);
+	if(fwrite(buff, 1, ret, fs) < ret) {
+	    logg("!Can't write to %s\n", file);
+	    free(file);
+	    return 58;
+	}
 
     fclose(fs);
 
@@ -164,9 +169,6 @@ int scanmanager(const struct optstruct *opt)
     }
 #endif
 
-    if(opt_check(opt, "ncore"))
-	dboptions |= CL_DB_NCORE;
-
     if(!opt_check(opt, "no-phishing-sigs"))
 	dboptions |= CL_DB_PHISHING;
 
@@ -186,6 +188,9 @@ int scanmanager(const struct optstruct *opt)
     if(opt_check(opt, "dev-ac-only"))
 	dboptions |= CL_DB_ACONLY;
 
+    if(opt_check(opt, "dev-ac-depth"))
+	cli_ac_setdepth(AC_DEFAULT_MIN_DEPTH, atoi(opt_arg(opt, "dev-ac-depth")));
+
     if(opt_check(opt, "detect-pua"))
 	dboptions |= CL_DB_PUA;
 
@@ -463,7 +468,10 @@ static int clamav_unpack(const char *prog, const char **args, const char *tmpdir
 		}
 	    }
 #endif
-	    chdir(tmpdir);
+	    if(chdir(tmpdir) == -1) {
+		fprintf(stderr, "ERROR: chdir(%s) failed\n", tmpdir);
+		exit(1);
+	    }
 
 	    if(printinfected) {
   	        fdevnull = open("/dev/null", O_WRONLY);
@@ -573,7 +581,7 @@ static void move_infected(const char *filename, const struct optstruct *opt)
     stat(filename, &ofstat);
 
     if(!stat(movefilename, &mfstat)) {
-        if(ofstat.st_ino == mfstat.st_ino) { /* It's the same file*/
+        if((ofstat.st_dev == mfstat.st_dev) && (ofstat.st_ino == mfstat.st_ino)) { /* It's the same file*/
             logg("File excluded '%s'\n", filename);
             info.notmoved++;
             free(movefilename);
