@@ -598,7 +598,7 @@ static int getfile(const char *srcfile, const char *destfile, const char *hostna
 
     /* check whether the resource actually existed or not */
     if((strstr(buffer, "HTTP/1.1 404")) != NULL || (strstr(buffer, "HTTP/1.0 404")) != NULL) { 
-	logg("!getfile: %s not found on remote server (IP: %s)\n", srcfile, ipaddr);
+	logg("^getfile: %s not found on remote server (IP: %s)\n", srcfile, ipaddr);
 	/* mirman_update(mdat->currip, mdat, 1); */
 	closesocket(sd);
 	return 58;
@@ -820,7 +820,7 @@ static struct cl_cvd *currentdb(const char *dbname, unsigned int *inc)
 {
 	struct stat sb;
 	char path[512];
-	struct cl_cvd *cvd;
+	struct cl_cvd *cvd = NULL;
 
 
     snprintf(path, sizeof(path), "%s.inc", dbname);
@@ -834,7 +834,8 @@ static struct cl_cvd *currentdb(const char *dbname, unsigned int *inc)
 	    *inc = 0;
     }
 
-    cvd = cl_cvdhead(path);
+    if(!access(path, R_OK))
+	cvd = cl_cvdhead(path);
 
     return cvd;
 }
@@ -1254,8 +1255,8 @@ int downloadmanager(const struct cfgstruct *copt, const struct optstruct *opt, c
 	if(arg) {
 	    if(opt_check(opt, "daemon"))
 		execute("OnUpdateExecute", arg);
-            else
-		system(arg);
+            else if(system(arg) == -1)
+		logg("!system(%s) failed\n", arg);
 	}
     }
 
@@ -1303,8 +1304,8 @@ int downloadmanager(const struct cfgstruct *copt, const struct optstruct *opt, c
 	    if(newver) {
 		if(opt_check(opt, "daemon"))
 		    execute("OnOutdatedExecute", cmd);
-		else
-		    system(cmd);
+		else if(system(cmd) == -1)
+		logg("!system(%s) failed\n", cmd);
 	    }
 	    free(cmd);
 	}
