@@ -165,13 +165,17 @@ int filecopy(const char *src, const char *dest)
     if((s = open(src, O_RDONLY|O_BINARY)) == -1)
 	return -1;
 
-    if((d = open(dest, O_CREAT|O_WRONLY|O_TRUNC|O_BINARY)) == -1) {
+    if((d = open(dest, O_CREAT|O_WRONLY|O_TRUNC|O_BINARY, 0644)) == -1) {
 	close(s);
 	return -1;
     }
 
     while((bytes = read(s, buffer, FILEBUFF)) > 0)
-	write(d, buffer, bytes);
+	if(write(d, buffer, bytes) < bytes) {
+	    close(s);
+	    close(d);
+	    return -1;
+	}
 
     close(s);
     /* njh@bandsman.co.uk: check result of close for NFS file */
